package cz.wz.milanl.colors.organizer;

import org.eclipse.jface.dialogs.IInputValidator;
import org.eclipse.jface.dialogs.InputDialog;
import org.eclipse.jface.window.Window;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.DisposeEvent;
import org.eclipse.swt.events.DisposeListener;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.RGB;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Dialog;
import org.eclipse.swt.widgets.List;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;

import cz.wz.milanl.colors.views.ColorView;

public class OrganizerWindow extends Dialog {
	private Organizer organizer;
	
	private List categories;
	private Table colors;
	
	private ColorView colorView;
	
	private Shell shell;
	
	public OrganizerWindow (Organizer organizer,Shell parent, int style, ColorView colorView) {
		super (parent, style);
		this.organizer=organizer;
		this.colorView=colorView;
		
		//Shell parent = getParent();
		shell = new Shell(parent, SWT.DIALOG_TRIM | SWT.APPLICATION_MODAL);
		buildShell(shell);
		
		shell.setLocation(parent.getBounds().x + (parent.getBounds().width - shell.getBounds().width) / 2,
		        parent.getBounds().y + (parent.getBounds().height -shell.getBounds().height) / 2);
	}
	
	public void open () {
		shell.open();
	}
	
	private void applyChanges()
	{
		organizer.save();
	}
	
	private void refreshColorList()
	{
		colors.removeAll();
		
		if (categories.getSelectionIndex()==-1) return;
		
		TableItem ti;
		for (int i=0;i<organizer.categories.get(categories.getSelectionIndex()).colors.size();i++)
		{
			ti=new TableItem(colors,0);
			ti.setText(0,organizer.categories.get(categories.getSelectionIndex()).colors.get(i).getName());
			ti.setBackground(1, new Color(null,organizer.categories.get(categories.getSelectionIndex()).colors.get(i).getRgb()));
			ti.setText(2,organizer.categories.get(categories.getSelectionIndex()).colors.get(i).getDescription());
		}
		
		for (int i=0;i<organizer.categories.get(categories.getSelectionIndex()).colors.size();i++)
		{
			if (compareRGB(colorView.getColor(),organizer.categories.get(categories.getSelectionIndex()).colors.get(i).getRgb()))
			{
				colors.setSelection(i);
				break;
			}
		}
	}
	
	private boolean compareRGB(RGB a,RGB b)
	{
		return a.red==b.red&&a.green==b.green&&a.blue==b.blue;
	}
	
	private void colorTableChangeItems(Table table, int a, int b)
	{
		String t=table.getItem(a).getText(0);
		table.getItem(a).setText(0, table.getItem(b).getText(0));
		table.getItem(b).setText(0,t);
		
		Color c=table.getItem(a).getBackground(1);
		table.getItem(a).setBackground(1, table.getItem(b).getBackground(1));
		table.getItem(b).setBackground(1,c);
		
		t=table.getItem(a).getText(2);
		table.getItem(a).setText(2, table.getItem(b).getText(2));
		table.getItem(b).setText(2,t);
	}
	
	private void buildShell(final Shell shell)
	{
		shell.addDisposeListener(new DisposeListener()
		{
			public void widgetDisposed(DisposeEvent e) {
				applyChanges();
			}
		});
		
		shell.setText("Color organizer");
		shell.setLayout(new GridLayout(3,false));
		
		GridData gd=null;
		
		categories=new List(shell,0);
		gd=new GridData();
		gd.widthHint=100;
		gd.heightHint=300;

		categories.setLayoutData(gd);
		categories.addSelectionListener(new SelectionListener()
		{
			public void widgetDefaultSelected(SelectionEvent e) {}

			public void widgetSelected(SelectionEvent e) {
				refreshColorList();				
			}
		});
		
		for (int i=0;i<organizer.categories.size();i++)
		{
			categories.add(organizer.categories.get(i).getName());
		}
		
		colors = new Table(shell,SWT.MULTI);
		
		gd=new GridData();
		gd.widthHint=350;
		gd.heightHint=300;
		colors.setLayoutData(gd);
		
		new TableColumn(colors,SWT.LEFT).setWidth(100);
		new TableColumn(colors,SWT.LEFT).setWidth(30);
		new TableColumn(colors,SWT.LEFT).setWidth(200);
		
		
		
		FillLayout fl=new FillLayout(SWT.VERTICAL);
		fl.spacing=5;
		
		Composite colorButtons=new Composite(shell,0);
		colorButtons.setLayout(fl);
		
		Button b_colorAdd=new Button(colorButtons,0);
		b_colorAdd.setText("Add color");
		b_colorAdd.addSelectionListener(new SelectionListener()
		{
			public void widgetDefaultSelected(SelectionEvent e) {}

			public void widgetSelected(SelectionEvent e) {
				if (categories.getSelectionIndex()==-1) return;
				InputDialog id = new InputDialog(shell,"Color name","Name:","New",new IInputValidator()
				{
					public String isValid(String newText) {
						if (newText.equals(""))
						{
							return "Invalid color name";
						}
						else if (organizer.categories.get(categories.getSelectionIndex()).hasColor(newText))
						{
							return "Color name already exists";
						}
						else return null;
					}
				});
				if (id.open()==Window.OK)
				{
					organizer.categories.get(categories.getSelectionIndex()).colors.add(new ColorItem(id.getValue(),"",new RGB(colorView.getColor().red,colorView.getColor().green,colorView.getColor().blue)));
					
					TableItem ti=new TableItem(colors,0);
					ti.setText(0,id.getValue());
					ti.setBackground(1, new Color(null,colorView.getColor()));
				}
				
			}
		});
		
		Button b_colorRename=new Button(colorButtons,0);
		b_colorRename.setText("Rename color");
		b_colorRename.addSelectionListener(new SelectionListener()
		{
			public void widgetDefaultSelected(SelectionEvent e) {}

			public void widgetSelected(SelectionEvent e) {
				if (colors.getSelectionIndex()==-1) return;
				final String originalName=organizer.categories.get(categories.getSelectionIndex()).colors.get(colors.getSelectionIndex()).getName();
				InputDialog id = new InputDialog(shell,"Color name","Name:",originalName,new IInputValidator()
				{
					public String isValid(String newText) {
						if (newText.equals(originalName))
						{
							return null;
						}
						else
						if (newText.equals(""))
						{
							return "Invalid color name";
						}
						else if (organizer.categories.get(categories.getSelectionIndex()).hasColor(newText))
						{
							return "Color already exists";
						}
						else return null;
					}
				});
				if (id.open()==Window.OK&&!id.getValue().equals(originalName))
				{
					organizer.categories.get(categories.getSelectionIndex()).colors.get(colors.getSelectionIndex()).setName(id.getValue());
					
					colors.getItem(colors.getSelectionIndex()).setText(0, id.getValue());
				}
			}
		});
		
		Button b_colorDelete=new Button(colorButtons,0);
		b_colorDelete.setText("Delete colors");
		b_colorDelete.addSelectionListener(new SelectionListener()
		{
			public void widgetDefaultSelected(SelectionEvent e) {}

			public void widgetSelected(SelectionEvent e) {
				if (colors.getSelectionIndex()==-1) return;
				
				MessageBox mb=new MessageBox(shell,SWT.YES|SWT.NO);
				mb.setText("Delete colors");
				mb.setMessage("Delete selected colors?");
				if (mb.open()==SWT.YES)
				{
					int[] indices=colors.getSelectionIndices();
					int d=0;
					for (int i=0;i<indices.length;i++)
					{
						organizer.categories.get(categories.getSelectionIndex()).colors.remove(indices[i]-d);
						colors.remove(indices[i]-d);
						d++;
					}
				}
			}
		});
		
		Button b_colorChange=new Button(colorButtons,0);
		b_colorChange.setText("Change colors");
		b_colorChange.addSelectionListener(new SelectionListener()
		{
			public void widgetDefaultSelected(SelectionEvent e) {}

			public void widgetSelected(SelectionEvent e) {
				if (colors.getSelectionIndex()==-1) return;
				
				MessageBox mb=new MessageBox(shell,SWT.YES|SWT.NO);
				mb.setText("Change colors");
				mb.setMessage("Change selected colors?");
				if (mb.open()==SWT.YES)
				{
					int[] indices=colors.getSelectionIndices();
					for (int i=0;i<indices.length;i++)
					{
						organizer.categories.get(categories.getSelectionIndex()).colors.get(indices[i]).setRgb(new RGB(colorView.getColor().red,colorView.getColor().green,colorView.getColor().blue));
						colors.getItem(indices[i]).setBackground(1, new Color(null,colorView.getColor()));
					}
				}
			}
		});
		
		Button b_colorDescription=new Button(colorButtons,0);
		b_colorDescription.setText("Change description");
		b_colorDescription.addSelectionListener(new SelectionListener()
		{
			public void widgetDefaultSelected(SelectionEvent e) {}

			public void widgetSelected(SelectionEvent e) {
				if (colors.getSelectionIndex()==-1) return;
				
				InputDialog id = new InputDialog(shell,"Color description","Description:",organizer.categories.get(categories.getSelectionIndex()).colors.get(colors.getSelectionIndex()).getDescription(),new IInputValidator()
				{
					public String isValid(String newText) {
						return null;
					}
				});
				if (id.open()==Window.OK)
				{
					int[] indices=colors.getSelectionIndices();
					for (int i=0;i<indices.length;i++)
					{
						organizer.categories.get(categories.getSelectionIndex()).colors.get(indices[i]).setDescription(id.getValue());
						colors.getItem(indices[i]).setText(2,id.getValue());
					}
				}
			}
		});
		
		Button b_colorUp=new Button(colorButtons,0);
		b_colorUp.setText("Move up");
		b_colorUp.addSelectionListener(new SelectionListener()
		{
			public void widgetDefaultSelected(SelectionEvent e) {}

			public void widgetSelected(SelectionEvent e) {
				int[] indices=colors.getSelectionIndices();
				for (int i=0;i<indices.length;i++)
				{
					if (indices[i]>0)
					{
						ColorItem ci=organizer.categories.get(categories.getSelectionIndex()).colors.get(indices[i]);
						organizer.categories.get(categories.getSelectionIndex()).colors.set(indices[i], organizer.categories.get(categories.getSelectionIndex()).colors.get(indices[i]-1));
						organizer.categories.get(categories.getSelectionIndex()).colors.set(indices[i]-1,ci);
						
						colorTableChangeItems(colors, indices[i], indices[i]-1);
						
						indices[i]--;
					}
				}
				colors.setSelection(indices);
			}
		});
		
		Button b_colorDown=new Button(colorButtons,0);
		b_colorDown.setText("Move down");
		b_colorDown.addSelectionListener(new SelectionListener()
		{
			public void widgetDefaultSelected(SelectionEvent e) {}

			public void widgetSelected(SelectionEvent e) {
				int[] indices=colors.getSelectionIndices();
				for (int i=0;i<indices.length;i++)
				{
					if (indices[i]<organizer.categories.get(categories.getSelectionIndex()).colors.size()-1&&indices[i]>-1)
					{
						ColorItem ci=organizer.categories.get(categories.getSelectionIndex()).colors.get(indices[i]);
						organizer.categories.get(categories.getSelectionIndex()).colors.set(indices[i], organizer.categories.get(categories.getSelectionIndex()).colors.get(indices[i]+1));
						organizer.categories.get(categories.getSelectionIndex()).colors.set(indices[i]+1,ci);
						
						colorTableChangeItems(colors, indices[i], indices[i]+1);
						
						indices[i]++;
					}
				}
				
				colors.setSelection(indices);
			}
		});
		
		Composite categoryButtons=new Composite(shell,0);
		categoryButtons.setLayout(fl);
		
		Button b_categoryAdd=new Button(categoryButtons,0);
		b_categoryAdd.setText("Add category");
		b_categoryAdd.addSelectionListener(new SelectionListener()
		{
			public void widgetDefaultSelected(SelectionEvent e) {}

			public void widgetSelected(SelectionEvent e) {
				InputDialog id = new InputDialog(shell,"Category name","Name:","New",new IInputValidator()
				{
					public String isValid(String newText) {
						if (newText.equals(""))
						{
							return "Invalid category name";
						}
						else if (organizer.hasCategory(newText))
						{
							return "Category already exists";
						}
						else return null;
					}
				});
				if (id.open()==Window.OK)
				{
					organizer.categories.add(new CategoryItem(id.getValue()));
					categories.add(id.getValue());
					categories.setSelection(categories.getItemCount()-1);
					refreshColorList();
				}
				
			}
		});
		
		Button b_categoryRename=new Button(categoryButtons,0);
		b_categoryRename.setText("Rename category");
		b_categoryRename.addSelectionListener(new SelectionListener()
		{
			public void widgetDefaultSelected(SelectionEvent e) {}

			public void widgetSelected(SelectionEvent e) {
				if (categories.getSelectionIndex()==-1) return;
				final String originalName=organizer.categories.get(categories.getSelectionIndex()).getName();
				InputDialog id = new InputDialog(shell,"Category name","Name:",originalName,new IInputValidator()
				{
					public String isValid(String newText) {
						if (newText.equals(originalName))
						{
							return null;
						}
						else
						if (newText.equals(""))
						{
							return "Invalid category name";
						}
						else if (organizer.hasCategory(newText))
						{
							return "Category already exists";
						}
						else return null;
					}
				});
				if (id.open()==Window.OK&&!id.getValue().equals(originalName))
				{
					organizer.categories.get(categories.getSelectionIndex()).setName(id.getValue());
					categories.setItem(categories.getSelectionIndex(), id.getValue());
				}
				
			}
		});
		
		Button b_categoryDelete=new Button(categoryButtons,0);
		b_categoryDelete.setText("Delete category");
		b_categoryDelete.addSelectionListener(new SelectionListener()
		{
			public void widgetDefaultSelected(SelectionEvent e) {}

			public void widgetSelected(SelectionEvent e) {
				if (categories.getSelectionIndex()==-1) return;
				
				MessageBox mb=new MessageBox(shell,SWT.YES|SWT.NO);
				mb.setText("Delete category");
				mb.setMessage("Delete?");
				if (mb.open()==SWT.YES)
				{
					organizer.categories.remove(categories.getSelectionIndex());
					categories.remove(categories.getSelectionIndex());
					refreshColorList();
				}
			}
		});
		
		Button b_categoryUp=new Button(categoryButtons,0);
		b_categoryUp.setText("Move up");
		b_categoryUp.addSelectionListener(new SelectionListener()
		{
			public void widgetDefaultSelected(SelectionEvent e) {}

			public void widgetSelected(SelectionEvent e) {
				int index=categories.getSelectionIndex();
				if (index>0)
				{
					CategoryItem ci=organizer.categories.get(index);
					organizer.categories.set(index, organizer.categories.get(index-1));
					organizer.categories.set(index-1,ci);
					
					String i=categories.getItem(index);
					categories.setItem(index, categories.getItem(index-1));
					categories.setItem(index-1, i);
					
					categories.setSelection(index-1);
				}
			}
		});
		
		Button b_categoryDown=new Button(categoryButtons,0);
		b_categoryDown.setText("Move down");
		b_categoryDown.addSelectionListener(new SelectionListener()
		{
			public void widgetDefaultSelected(SelectionEvent e) {}

			public void widgetSelected(SelectionEvent e) {
				int index=categories.getSelectionIndex();
				if (index<organizer.categories.size()-1&&index>-1)
				{
					CategoryItem ci=organizer.categories.get(index);
					organizer.categories.set(index, organizer.categories.get(index+1));
					organizer.categories.set(index+1,ci);
					
					String i=categories.getItem(index);
					categories.setItem(index, categories.getItem(index+1));
					categories.setItem(index+1, i);
					
					categories.setSelection(index+1);
				}
			}
		});
		
		Composite buttons=new Composite(shell,0);
		buttons.setLayout(fl);
		
		Button b_use=new Button(buttons,0);
		b_use.setText("Use color");
		b_use.addSelectionListener(new SelectionListener()
		{
			public void widgetDefaultSelected(SelectionEvent e) {}

			public void widgetSelected(SelectionEvent e) {
				if (colors.getSelectionIndex()==-1) return;
				RGB rgb=organizer.categories.get(categories.getSelectionIndex()).colors.get(colors.getSelectionIndex()).getRgb();
				colorView.changeColor(new RGB(rgb.red,rgb.green,rgb.blue), null);
				shell.close();
			}
		});
		
		Button b_close=new Button(buttons,0);
		b_close.setText("Close");
		b_close.addSelectionListener(new SelectionListener()
		{
			public void widgetDefaultSelected(SelectionEvent e) {}

			public void widgetSelected(SelectionEvent e) {
				shell.close();
			}
		});
		
		
		
		shell.pack();
	}

}