package cz.wz.milanl.colors.selectors;

import org.eclipse.swt.graphics.RGB;

import cz.wz.milanl.util.Numbers;

public class CMYKSelector extends NumericValuesSelector {

	public CMYKSelector(SelectorBar parent, String name) {
		super(parent, name, new NumericValue[] { new NumericValue("C", 0, 1000),
				new NumericValue("M", 0, 1000), new NumericValue("Y", 0, 1000),new NumericValue("K", 0, 1000), });
	}

	protected void insertValueToPrimary(RGB rgb) {
		float c=(float)1-((float)rgb.red)/(float)255;
		float m=(float)1-((float)rgb.green)/(float)255;
		float y=(float)1-((float)rgb.blue)/(float)255;
		
		float k=1;
		if (c<k) k=c;
		if (m<k) k=m;
		if (y<k) k=y;
		
		if (k==1)
		{
			c=0;
			m=0;
			y=0;
			
		}
		else
		{
			c=(c-k)/((float)1-k);
			m=(m-k)/((float)1-k);
			y=(y-k)/((float)1-k);
		}
		texts[0].setText("" + (int)(c*1000));
		texts[1].setText("" + (int)(m*1000));
		texts[2].setText("" + (int)(y*1000));
		texts[3].setText("" + (int)(k*1000));
	}

	protected RGB getValue() {
		float c=((float)Numbers.toNumber(texts[0].getText()))/(float)1000;
		float m=((float)Numbers.toNumber(texts[1].getText()))/(float)1000;
		float y=((float)Numbers.toNumber(texts[2].getText()))/(float)1000;
		float k=((float)Numbers.toNumber(texts[3].getText()))/(float)1000;
		c=c*((float)1 - k) + k;
		m=m*((float)1 - k) + k;
		y=y*((float)1 - k) + k;

		int r=(int)(((float)1-c)*(float)255);
		int g=(int)(((float)1-m)*(float)255);
		int b=(int)(((float)1-y)*(float)255);
		return new RGB(r,g,b); 
	}

	public String getCopyText(int index) {
		switch (index)
		{
		case 0: return texts[0].getText() + "," + texts[1].getText() + ","
		+ texts[2].getText()+ ","+ texts[3].getText();
		case 1: return texts[0].getText() + ";" + texts[1].getText() + ";"
		+ texts[2].getText()+ ";"+ texts[3].getText();
		case 2: return texts[0].getText() + " " + texts[1].getText() + " "
		+ texts[2].getText()+ " "+ texts[3].getText();
		case 3: return "cmyk(" + texts[0].getText() + "," + texts[1].getText()
		+ "," + texts[2].getText() + "," + texts[3].getText()+ ")";
		case 4: return "CMYK(" + texts[0].getText() + "," + texts[1].getText()
		+ "," + texts[2].getText() + "," + texts[3].getText() + ")";
		case 5: return "cmyk (" + texts[0].getText() + "," + texts[1].getText()
		+ "," + texts[2].getText() + "," + texts[3].getText()+ ")";
		case 6: return "CMYK (" + texts[0].getText() + "," + texts[1].getText()
		+ "," + texts[2].getText() + "," + texts[3].getText()+ ")";
		case 7: return "cmyk(" + texts[0].getText() + ";" + texts[1].getText()
		+ ";" + texts[2].getText() + ";" + texts[3].getText() + ")";
		case 8: return "CMYK(" + texts[0].getText() + ";" + texts[1].getText()
		+ ";" + texts[2].getText() + ";" + texts[3].getText()+ ")";
		case 9: return "cmyk (" + texts[0].getText() + ";" + texts[1].getText()
		+ ";" + texts[2].getText() + ";" + texts[3].getText()+ ")";
		case 10: return "CMYK (" + texts[0].getText() + ";" + texts[1].getText()
		+ ";" + texts[2].getText() + ";" + texts[3].getText()+ ")";
		case 11: return "cmyk(" + texts[0].getText() + " " + texts[1].getText()
		+ " " + texts[2].getText() + " " + texts[3].getText()+ ")";
		case 12: return "CMYK(" + texts[0].getText() + " " + texts[1].getText()
		+ " " + texts[2].getText() + " " + texts[3].getText()+ ")";
		case 13: return "cmyk (" + texts[0].getText() + " " + texts[1].getText()
		+ " " + texts[2].getText() + " " + texts[3].getText()+ ")";
		case 14: return "CMYK (" + texts[0].getText() + " " + texts[1].getText()
		+ " " + texts[2].getText() + " " + texts[3].getText() +")";
		case 15: return "c:" + texts[0].getText() + ",m:" + texts[1].getText()
		+ ",y:" + texts[2].getText() + ",k:" + texts[3].getText();
		case 16: return "C:" + texts[0].getText() + ",M:" + texts[1].getText()
		+ ",Y:" + texts[2].getText() + ",K:" + texts[3].getText();
		case 17: return "c:" + texts[0].getText() + ";m:" + texts[1].getText()
		+ ";y:" + texts[2].getText() + ";k:" + texts[3].getText();
		case 18: return "C:" + texts[0].getText() + ";M:" + texts[1].getText()
		+ ";Y:" + texts[2].getText()+ ";K:" + texts[3].getText();
		case 19: return "c:" + texts[0].getText() + " m:" + texts[1].getText()
		+ " y:" + texts[2].getText()+ " k:" + texts[3].getText();
		case 20: return "C:" + texts[0].getText() + " M:" + texts[1].getText()
		+ " Y:" + texts[2].getText() + " K:" + texts[3].getText();
		case 21: return "c:" + texts[0].getText() + "m:" + texts[1].getText()
		+ "y:" + texts[2].getText()+ "k:" + texts[3].getText();
		case 22: return "C:" + texts[0].getText() + "M:" + texts[1].getText()
		+ "Y:" + texts[2].getText()+ "K:" + texts[3].getText();
		case 23: return "c" + texts[0].getText() + ",m" + texts[1].getText()
		+ ",y" + texts[2].getText()+ ",k" + texts[3].getText();
		case 24: return "C" + texts[0].getText() + ",M" + texts[1].getText()
		+ ",Y" + texts[2].getText()+ ",K" + texts[3].getText();
		case 25: return "c" + texts[0].getText() + ";m" + texts[1].getText()
		+ ";y" + texts[2].getText()+ ";k" + texts[3].getText();
		case 26: return "C" + texts[0].getText() + ";M" + texts[1].getText()
		+ ";Y" + texts[2].getText()+ ";K" + texts[3].getText();
		case 27: return "c" + texts[0].getText() + " m" + texts[1].getText()
		+ " y" + texts[2].getText()+ " k" + texts[3].getText();
		case 28: return "C" + texts[0].getText() + " M" + texts[1].getText()
		+ " Y" + texts[2].getText()+ " K" + texts[3].getText();
		case 29: return "c" + texts[0].getText() + "m" + texts[1].getText()
		+ "y" + texts[2].getText()+ "k" + texts[3].getText();
		case 30: return "C" + texts[0].getText() + "M" + texts[1].getText()
		+ "Y" + texts[2].getText()+ "K" + texts[3].getText();
		}
		
		return null;
	}
}
