package cz.wz.milanl.colors.selectors;

import java.awt.Toolkit;
import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.FlavorEvent;
import java.awt.datatransfer.FlavorListener;

import org.eclipse.jface.text.ITextSelection;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.*;
import org.eclipse.swt.layout.*;
import org.eclipse.swt.widgets.*;
import org.eclipse.ui.ISelectionListener;
import org.eclipse.ui.IWorkbenchPart;

import cz.wz.milanl.util.Icons;

public abstract class ColorSelector extends Composite {
	private SelectorBar selectorBar;

	protected Color errorColor = new Color(null, 255, 200, 200);
	protected Color defaultColor = new Color(null, 255, 255, 255);
	private Color toolBarBackground = new Color(null, 217, 217, 255);

	private Composite toolArea;

	private ToolBar nameToolbar;
	private ToolItem nameItem;

	private ToolBar toolbar;
	private ToolItem pasteItem;
	private ToolItem copyItem;
	private Menu copyMenu;
	private ToolItem pasteSourceItem;
	private ToolItem copySourceItem;
	private ToolItem autoItem;
	private Menu autoMenu;
	private MenuItem[] autoMenuItems;

	private ToolBar textsToolbar;
	private ToolItem textsMenuItem;
	private MenuItem[] textMenuItems;
	private int selectedTextMenuItem;
	private boolean copyAction;
	
	protected Composite area;

	private boolean expand;

	private Clipboard clipboard;
	private FlavorListener flavorListener;
	private ISelectionListener sourceSelectionListener;
	
	private boolean autoPasteFromClipboardEnabled;
	private boolean autoCopyToClipboardEnabled;
	private boolean autoPasteFromSourceEnabled;
	private boolean autoCopyToSourceEnabled;

	private boolean primaryListenerEnabled;
	private boolean secondaryListenerEnabled;
	
	
	public ColorSelector(final SelectorBar selectorBar, String name) {
		super(selectorBar, 0);

		this.selectorBar = selectorBar;

		setLayout(new FormLayout());

		toolArea = new Composite(this, 0);
		FormData data = new FormData();
		data.top = new FormAttachment(0, 0);
		data.left = new FormAttachment(0);
		data.right = new FormAttachment(100);
		toolArea.setLayoutData(data);
		RowLayout rl = new RowLayout();
		rl.marginLeft = 0;
		rl.marginTop = 0;
		rl.marginRight = 0;
		rl.marginBottom = 0;
		rl.wrap = false;
		rl.spacing = 0;
		toolArea.setLayout(rl);
		toolArea.setBackground(toolBarBackground);

		nameToolbar = new ToolBar(toolArea, SWT.FLAT);
		nameToolbar.setBackground(toolBarBackground);
		nameItem = new ToolItem(nameToolbar, SWT.PUSH);
		nameItem.setToolTipText("Show/hide");
		nameItem.setText(name);
		nameItem.addSelectionListener(new SelectionListener() {
			public void widgetDefaultSelected(SelectionEvent e) {

			}

			public void widgetSelected(SelectionEvent e) {
				showHide();
			}

		});

		toolbar = new ToolBar(toolArea, SWT.FLAT);
		toolbar.setBackground(toolBarBackground);
		pasteItem = new ToolItem(toolbar, SWT.PUSH);
		pasteItem.setImage(Icons.get("from_clipboard"));
		pasteItem.setToolTipText("Insert from clipboard");
		pasteItem.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event event) {
				paste(selectorBar.colors.getFromClipboard());
			}
		});
		copyItem = new ToolItem(toolbar, SWT.PUSH);
		copyItem.setImage(Icons.get("to_clipboard"));
		copyItem.setToolTipText("Copy to clipboard");
		copyItem.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event event) {
				selectorBar.colors.copyToClipboard(textsMenuItem.getText());
			}

		});
		pasteSourceItem = new ToolItem(toolbar, SWT.PUSH);
		pasteSourceItem.setImage(Icons.get("from_source"));
		pasteSourceItem.setToolTipText("Insert from source code");
		pasteSourceItem.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event event) {
				paste(selectorBar.colors.getSourceSelection());
			}
		});
		copySourceItem = new ToolItem(toolbar, SWT.PUSH);
		copySourceItem.setImage(Icons.get("to_source"));
		copySourceItem.setToolTipText("Copy to source code");
		copySourceItem.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event event) {
				selectorBar.colors.copyToSource(textsMenuItem.getText());
			}
		});
		autoItem = new ToolItem(toolbar, SWT.PUSH);
		autoItem.setImage(Icons.get("options"));
		autoItem.setToolTipText("Automatic actions");
		autoItem.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event e) {
				if (autoMenu == null)
					createAutoMenu();
				else
					refreshAutoMenuCheck();
				Rectangle rect = autoItem.getBounds();
				Point pt = new Point(rect.x, rect.y + rect.height);
				pt = toolbar.toDisplay(pt);
				autoMenu.setLocation(pt.x, pt.y);
				autoMenu.setVisible(true);
			}
		});

		textsToolbar = new ToolBar(toolArea, SWT.FLAT);
		textsToolbar.setBackground(toolBarBackground);
		textsMenuItem = new ToolItem(textsToolbar, SWT.PUSH);
		textsMenuItem.setToolTipText("Format selection");
		textsMenuItem.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event e) {
				if (copyMenu == null)
					createCopyMenu();
				else
					refreshCopyMenuTexts();
				Rectangle rect = textsMenuItem.getBounds();
				Point pt = new Point(rect.x, rect.y + rect.height);
				pt = textsToolbar.toDisplay(pt);
				copyMenu.setLocation(pt.x, pt.y);
				copyMenu.setVisible(true);
			}
		});

		toolArea.pack();

		area = new Composite(this, 0);
		data = new FormData();
		data.top = new FormAttachment(0, toolbar.getSize().y);
		data.left = new FormAttachment(0);
		data.right = new FormAttachment(100);
		area.setLayoutData(data);

		
	}

	void setAutoPasteFromClipboard(boolean enabled) {
		if (enabled == true) {
			if (autoPasteFromClipboardEnabled == false) {
				selectorBar.disableAutoClipboardActions();
				selectorBar.colors.clearClipboard();
				
				clipboard=Toolkit.getDefaultToolkit().getSystemClipboard();
				flavorListener=new FlavorListener()
				{
					public void flavorsChanged(FlavorEvent arg0) {
						Display.getDefault().asyncExec(new Runnable() {
							   public void run() {
									try {
										//String s=(String)clipboard.getData(DataFlavor.stringFlavor);
										
										String s=selectorBar.colors.getFromClipboard();
										if (s!=null)
										{
											clipboard.removeFlavorListener(flavorListener);
											
											selectorBar.colors.clearClipboard();
											paste(s);
											
											clipboard.addFlavorListener(flavorListener);
										}
										
										
									} catch (Exception e) {}
							   }
							});
					}
				};
				clipboard.addFlavorListener(flavorListener);
				autoPasteFromClipboardEnabled = true;
				setAutoItemImage();
			}
		} else {
			if (autoPasteFromClipboardEnabled == true) {
				clipboard.removeFlavorListener(flavorListener);
				clipboard=null;
				flavorListener=null;
				autoPasteFromClipboardEnabled = false;
				setAutoItemImage();
			}
		}
	}

	void setAutoCopyToClipboard(boolean enabled) {
		if (enabled)
			selectorBar.disableAutoClipboardActions();
		autoCopyToClipboardEnabled = enabled;
		setAutoItemImage();
	}
	
	void setAutoPasteFromSource(boolean enabled) {
		if (enabled == true) {
			if (autoPasteFromSourceEnabled == false) {
				selectorBar.disableAutoSourceActions();
				sourceSelectionListener = new ISelectionListener() {

					public void selectionChanged(IWorkbenchPart part,
							ISelection selection) {
						if (part == selectorBar.colors.getViewSite()
								.getWorkbenchWindow().getActivePage()
								.getActiveEditor()
								&& selection instanceof ITextSelection) {
							String sel = ((ITextSelection) selection).getText();
							if (!sel.equals(""))
								paste(sel);
						}
					}
				};
				selectorBar.colors.getViewSite().getWorkbenchWindow()
						.getActivePage().addSelectionListener(
								sourceSelectionListener);

				autoPasteFromSourceEnabled = true;
				setAutoItemImage();
			}
		} else {
			if (autoPasteFromSourceEnabled == true) {
				selectorBar.colors.getViewSite().getWorkbenchWindow()
						.getActivePage().removeSelectionListener(
								sourceSelectionListener);
				sourceSelectionListener=null;
				
				autoPasteFromSourceEnabled = false;
				setAutoItemImage();
			}
		}

	}

	void setAutoCopyToSource(boolean enabled) {
		if (enabled)
			selectorBar.disableAutoSourceActions();
		autoCopyToSourceEnabled = enabled;
		setAutoItemImage();
	}

	private void setAutoItemImage() {
		Image i = (autoPasteFromClipboardEnabled || autoCopyToClipboardEnabled
				|| autoPasteFromSourceEnabled || autoCopyToSourceEnabled) ? Icons
				.get("options_active")
				: Icons.get("options");
		if (autoItem.getImage() != i) {
			autoItem.setImage(i);
		}
	}

	private void autoCopyToClipboard() {
		if (autoCopyToClipboardEnabled) {
			selectorBar.colors.copyToClipboard(textsMenuItem.getText());
		}
	}

	private void autoCopyToSource() {
		if (autoCopyToSourceEnabled) {
			selectorBar.colors.copyToSource(textsMenuItem.getText());
		}
	}

	private void createCopyMenu() {
		if (getCopyText(0) == null)
			return;
		copyMenu = new Menu(selectorBar.getShell(), SWT.POP_UP);
		
		int length=0;
		String s=getCopyText(length);
		while (s!=null)
		{
			length++;
			s=getCopyText(length);
		}
		
		textMenuItems = new MenuItem[length];
		for (int i = 0; i < length; i++) {
			final int item = i;
			textMenuItems[i] = new MenuItem(copyMenu, SWT.PUSH);
			textMenuItems[i].addSelectionListener(new SelectionListener() {
				public void widgetDefaultSelected(SelectionEvent e) {
				}

				public void widgetSelected(SelectionEvent e) {
					if (!copyAction) {
						selectorBar.colors.copyToClipboard(((MenuItem) e
								.getSource()).getText());

					} else {
						selectorBar.colors.copyToSource(((MenuItem) e
								.getSource()).getText());

					}
					selectAutoMenuItem(item);
				}

			});
		}
		refreshCopyMenuTexts();
	}

	private void refreshCopyMenuTexts() {
		int i=0;
		String s=getCopyText(i);
		while (s!=null)
		{
			textMenuItems[i].setText(s);
			i++;
			s=getCopyText(i);
		}
	}

	private void createAutoMenu() {
		autoMenu = new Menu(selectorBar.getShell(), SWT.POP_UP);
		autoMenuItems = new MenuItem[4];
		for (int i = 0; i < 4; i++) {
			final int item = i;
			autoMenuItems[i] = new MenuItem(autoMenu, SWT.CHECK);
			switch (item) {
			case 0:
				autoMenuItems[i]
						.setText("Insert from clipboard on clipboard change");
				break;
			case 1:
				autoMenuItems[i].setText("Copy to clipboard on change");
				break;
			case 2:
				autoMenuItems[i]
						.setText("Insert from source on selection change");
				break;
			case 3:
				autoMenuItems[i].setText("Copy to source on change");
				break;
			}
			autoMenuItems[i].addSelectionListener(new SelectionListener() {
				public void widgetDefaultSelected(SelectionEvent e) {
				}

				public void widgetSelected(SelectionEvent e) {
					switch (item) {
					case 0:
						setAutoPasteFromClipboard(!autoPasteFromClipboardEnabled);
						break;
					case 1:
						setAutoCopyToClipboard(!autoCopyToClipboardEnabled);
						break;
					case 2:
						setAutoPasteFromSource(!autoPasteFromSourceEnabled);
						break;
					case 3:
						setAutoCopyToSource(!autoCopyToSourceEnabled);
						break;
					}
				}

			});
		}
		refreshAutoMenuCheck();
	}

	private void refreshAutoMenuCheck() {
		autoMenuItems[0].setSelection(autoPasteFromClipboardEnabled);
		autoMenuItems[1].setSelection(autoCopyToClipboardEnabled);
		autoMenuItems[2].setSelection(autoPasteFromSourceEnabled);
		autoMenuItems[3].setSelection(autoCopyToSourceEnabled);
	}

	private void selectAutoMenuItem(int item) {
		selectedTextMenuItem = item;
		ioActions();
	}

	private void refreshCopyItemText() {
		if (getCopyText(0) == null)
			return;
		textsMenuItem.setText(getCopyText(selectedTextMenuItem));
		textsMenuItem.setEnabled(true);
		copyItem.setEnabled(true);
		copySourceItem.setEnabled(true);
		textsToolbar.pack();
	}

	private void disableCopyItem() {
		textsMenuItem.setText("");
		textsMenuItem.setEnabled(false);
		copyItem.setEnabled(false);
		copySourceItem.setEnabled(false);
		textsToolbar.pack();
	}

	void expand() {
		expand = true;
		setSize(getSize().x, getHeight());
	}

	void collapse() {
		expand = false;
		setSize(getSize().x, getHeight());
	}

	void showHide() {
		if (expand)
			collapse();
		else
			expand();
		selectorBar.refreshPositions();
	}

	int getHeight() {
		int height = toolbar.getSize().y;
		if (expand)
			height += area.getSize().y;
		return height;
	}

	protected abstract void insertValueToPrimary(RGB rgb);

	protected abstract RGB getValue();

	protected abstract void insertTextToPrimary(String text);

	protected abstract void refreshPrimaryIndicatorFields();

	protected abstract void refreshSecondaryIndicatorFields();

	protected abstract boolean checkAllValues(boolean changeFieldColor);

	protected abstract void restoreFieldColors();

	protected abstract String getCopyText(int index);

	private void actionOnPrimaryChange() {
		if (!primaryListenerEnabled)
			return;
		refreshSecondaryIndicators();
		colorOut();
	}

	private void actionOnSecondaryChange() {
		if (!secondaryListenerEnabled)
			return;
		refreshPrimaryIndicators();
		colorOut();
	}

	private void insertText(String text) {
		primaryListenerEnabled = false;
		insertTextToPrimary(text);
		primaryListenerEnabled = true;
	}

	private void insertValue(RGB rgb) {
		primaryListenerEnabled = false;
		insertValueToPrimary(rgb);
		primaryListenerEnabled = true;
	}

	private void refreshPrimaryIndicators() {
		primaryListenerEnabled = false;
		refreshPrimaryIndicatorFields();
		primaryListenerEnabled = true;
	}

	private void refreshSecondaryIndicators() {
		secondaryListenerEnabled = false;
		refreshSecondaryIndicatorFields();
		secondaryListenerEnabled = true;
	}

	void colorIn() {
		insertValue(selectorBar.colorView.getColor());

		refreshSecondaryIndicators();

		if (checkAllValues(true)) {
			ioActions();
		} else {
			disableCopyItem();
		}
	}

	private void colorOut() {
		if (checkAllValues(true)) {
			selectorBar.colorView.changeColor(getValue(), this);
			ioActions();
		} else {
			disableCopyItem();
		}
	}

	private void paste(String text) {
		insertText(text);

		refreshSecondaryIndicators();
		colorOut();
	}

	private void ioActions() {
		refreshCopyItemText();
		autoCopyToClipboard();
		autoCopyToSource();
	}

	protected ModifyListener getModifyListener(final boolean secondary) {
		return new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				if (secondary)
					actionOnSecondaryChange();
				else
					actionOnPrimaryChange();

			}
		};
	}

	protected SelectionListener getSelectionListener(final boolean secondary) {
		return new SelectionListener() {
			public void widgetDefaultSelected(SelectionEvent e) {
			}

			public void widgetSelected(SelectionEvent e) {
				if (secondary)
					actionOnSecondaryChange();
				else
					actionOnPrimaryChange();
			}
		};
	}
}
