package cz.wz.milanl.colors.selectors;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.*;

public class EditSelectorsWindow extends Dialog {

	private int[] indices;
	private Table table;
	
	SelectorManager selectorManager;
	
	public EditSelectorsWindow (SelectorManager selectorManager,Shell parent, int style) {
		super (parent, style);
		this.selectorManager=selectorManager;
		this.indices=new int[selectorManager.indices.length];
		for (int i=0;i<selectorManager.indices.length;i++)
		{
			this.indices[i]=selectorManager.indices[i];
		}
	}
	
	public void open () {
		Shell parent = getParent();
		Shell shell = new Shell(parent, SWT.DIALOG_TRIM | SWT.APPLICATION_MODAL);
		buildShell(shell);
		
		shell.setLocation(parent.getBounds().x + (parent.getBounds().width - shell.getBounds().width) / 2,
		        parent.getBounds().y + (parent.getBounds().height -shell.getBounds().height) / 2);
        
		shell.open();
	}
	
	private void applyChanges()
	{
		selectorManager.indices=indices;
		
		for (int i=0;i<indices.length;i++)
		{
			boolean ic=table.getItem(i).getChecked();
			selectorManager.sel[indices[i]]=ic;
		}
		
		selectorManager.save();
		selectorManager.applyToSelectorBar();
	}
	
	private void buildShell(final Shell shell)
	{
		shell.setText("Edit color selectors");
		shell.setLayout(new GridLayout(2,false));

		table = new Table(shell, SWT.CHECK|SWT.FULL_SELECTION);
		TableItem tableItem=null;
		for (int i=0;i<selectorManager.indices.length;i++)
		{
			tableItem=new TableItem(table,SWT.NONE);
			tableItem.setChecked(selectorManager.sel[selectorManager.indices[i]]);
			tableItem.setText(SelectorManager.allItems[selectorManager.indices[i]].getName());
		}
		
		Composite buttons=new Composite(shell,0);
		FillLayout fl=new FillLayout(SWT.VERTICAL);
		fl.spacing=5;
		buttons.setLayout(fl);
		
		Button b_ok=new Button(buttons,0);
		b_ok.setText("OK");
		b_ok.addSelectionListener(new SelectionListener()
		{
			public void widgetDefaultSelected(SelectionEvent e) {}

			public void widgetSelected(SelectionEvent e) {
				applyChanges();
				shell.close();
			}
		});
		
		Button b_cancel=new Button(buttons,0);
		b_cancel.setText("Cancel");
		b_cancel.addSelectionListener(new SelectionListener()
		{
			public void widgetDefaultSelected(SelectionEvent e) {}

			public void widgetSelected(SelectionEvent e) {
				shell.close();
			}
		});
		
		Button b_up=new Button(buttons,0);
		b_up.setText("Move up");
		b_up.addSelectionListener(new SelectionListener()
		{
			public void widgetDefaultSelected(SelectionEvent e) {}

			public void widgetSelected(SelectionEvent e) {
				int index=table.getSelectionIndex();
				if (index>0)
				{
					int i=indices[index];
					indices[index]=indices[index-1];
					indices[index-1]=i;
					
					tableChangeItems(table, index, index-1);
					table.setSelection(index-1);
				}
			}
		});
		
		Button b_down=new Button(buttons,0);
		b_down.setText("Move down");
		b_down.addSelectionListener(new SelectionListener()
		{
			public void widgetDefaultSelected(SelectionEvent e) {}

			public void widgetSelected(SelectionEvent e) {
				int index=table.getSelectionIndex();
				if (index<indices.length-1&&index>-1)
				{
					int i=indices[index];
					indices[index]=indices[index+1];
					indices[index+1]=i;
					
					tableChangeItems(table, index, index+1);
					table.setSelection(index+1);
				}
			}
		});
		
		shell.pack();
		
	}
	
	private void tableChangeItems(Table table, int a, int b)
	{
		String t=table.getItem(a).getText();
		boolean ch=table.getItem(a).getChecked();
		
		table.getItem(a).setText(table.getItem(b).getText());
		table.getItem(a).setChecked(table.getItem(b).getChecked());
		
		table.getItem(b).setText(t);
		table.getItem(b).setChecked(ch);
	}
}
