package cz.wz.milanl.colors.selectors;

import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.RGB;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.FormAttachment;
import org.eclipse.swt.layout.FormData;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;

public abstract class ListSelector extends ColorSelector {
	ColorListItem[] colorItems;
	
	Table table;

	public ListSelector(SelectorBar parent, String name) {
		super(parent, name);
		area.setLayout(new FillLayout());

		table = new Table(area,0);
		new TableColumn(table,SWT.LEFT).setWidth(100);
		new TableColumn(table,SWT.LEFT).setWidth(30);
		table.addSelectionListener(getSelectionListener(false));

		colorItems = getColorListItems();
		
		TableItem ti;
		for (int i = 0; i < colorItems.length; i++) {
			ti=new TableItem(table,0);
			ti.setText(0,colorItems[i].name);
			ti.setBackground(1,new Color(null,new RGB(colorItems[i].r,colorItems[i].g,colorItems[i].b)));
		}

		((FormData) area.getLayoutData()).bottom = new FormAttachment(0, 100);
	}

	protected abstract ColorListItem[] getColorListItems();

	protected boolean checkAllValues(boolean changeFieldColor) {
		return table.getSelectionIndex() > -1;
	}

	protected RGB getValue() {
		ColorListItem sel = colorItems[table.getSelectionIndex()];
		return new RGB(sel.r, sel.g, sel.b);
	}

	protected void insertTextToPrimary(String text) {
		int sel = -1;
		int max = 0;
		int e = 0;
		text = text.toLowerCase();
		for (int i = 0; i < colorItems.length; i++) {
			e = equalChars(colorItems[i].name.toLowerCase(), text);
			if (e > max) {
				max = e;
				sel = i;
			}
		}
		table.setSelection(sel);
	}

	private int equalChars(String a, String b) {
		int length = (a.length() > b.length()) ? b.length() : a.length();
		for (int i = 0; i < length; i++) {
			if (a.charAt(i) != b.charAt(i) || i == length - 1)
				return i;
		}

		return 0;
	}

	protected void insertValueToPrimary(RGB rgb) {
		boolean ok = false;
		for (int i = 0; i < colorItems.length; i++) {
			if (rgb.red == colorItems[i].r && rgb.green == colorItems[i].g
					&& rgb.blue == colorItems[i].b) {
				table.setSelection(i);
				ok = true;
				break;
			}
		}
		if (ok == false) {
			table.setSelection(-1);
		}
	}

	protected void refreshPrimaryIndicatorFields() {
	}

	protected void refreshSecondaryIndicatorFields() {
	}

	public void restoreFieldColors() {
	}

	public String getCopyText(int index) {
		switch (index)
		{
			case 0: return colorItems[table.getSelectionIndex()].name;
			case 1: return colorItems[table.getSelectionIndex()].name.toLowerCase();
			case 2: return colorItems[table.getSelectionIndex()].name.toUpperCase();
		}
		return null;
	}
}