package cz.wz.milanl.colors.selectors;

import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Slider;
import org.eclipse.swt.widgets.Text;

import cz.wz.milanl.util.Numbers;

public abstract class NumericValuesSelector extends ColorSelector {
	private NumericValue[] values;

	private Label[] labels;
	protected Text[] texts;
	private Slider[] sliders;

	public NumericValuesSelector(SelectorBar parent, String name,
			NumericValue[] values) {
		super(parent, name);
		this.values = values;
		GridLayout gl = new GridLayout(3, false);
		gl.horizontalSpacing = 1;
		gl.verticalSpacing = 1;
		area.setLayout(gl);

		labels = new Label[values.length];
		texts = new Text[values.length];
		sliders = new Slider[values.length];
		for (int i = 0; i < values.length; i++) {
			labels[i] = new Label(area, 0);
			labels[i].setText(values[i].getName());
			texts[i] = new Text(area, 0);
			texts[i].addModifyListener(getModifyListener(false));
			sliders[i] = new Slider(area, 0);
			sliders[i].setMinimum(values[i].getMin());
			sliders[i].setMaximum(values[i].getMax() + 10);
			sliders[i].addSelectionListener(getSelectionListener(true));
			GridData gd = new GridData();
			gd.grabExcessHorizontalSpace = true;
			gd.widthHint = 10000;
			sliders[i].setLayoutData(gd);
		}
	}

	protected void refreshSecondaryIndicatorFields() {
		for (int i = 0; i < values.length; i++) {
			if (checkNumericValue(i))
				sliders[i].setSelection(Numbers.toNumber(texts[i].getText()));
		}
	}

	protected void refreshPrimaryIndicatorFields() {
		for (int i = 0; i < values.length; i++) {
			texts[i].setText("" + sliders[i].getSelection());
		}
	}

	private boolean checkNumericValue(int index) {
		if (texts[index].getText().length()==0||!Numbers.isNumeric(texts[index].getText())
				|| Numbers.toNumber(texts[index].getText()) < values[index]
						.getMin()
				|| Numbers.toNumber(texts[index].getText()) > values[index]
						.getMax()) {
			return false;
		} else {
			return true;
		}
	}

	protected void insertTextToPrimary(String text) {
		int[] numbers = Numbers.getFirstNumbers(text, values.length, Integer.MIN_VALUE);
		for (int i=0;i<values.length;i++)
		{
			if (numbers[i]!=Integer.MIN_VALUE) texts[i].setText("" + numbers[i]); else texts[i].setText("");
		}
	}

	protected boolean checkAllValues(boolean changeFieldColor) {
		boolean result = true;
		for (int i = 0; i < values.length; i++) {
			if (!checkNumericValue(i)) {
				if (changeFieldColor)
					texts[i].setBackground(errorColor);
				result = false;
			} else {
				if (changeFieldColor)
					texts[i].setBackground(defaultColor);
			}
		}
		return result;
	}

	public void restoreFieldColors() {
		for (int i = 0; i < values.length; i++) {
			texts[i].setBackground(defaultColor);
		}
	}
}
