package cz.wz.milanl.colors.selectors;

import java.io.IOException;
import java.lang.reflect.Constructor;

import org.eclipse.core.runtime.IPath;
import org.eclipse.jface.dialogs.DialogSettings;

import cz.wz.milanl.colors.Activator;
public class SelectorManager {
	public static SelectorItem[] allItems=new SelectorItem[]
	{
			new SelectorItem(RGBSelector.class,"RGB"),
			new SelectorItem(HTMLCodeSelector.class,"HTML code"),
			new SelectorItem(HTMLNameSelector.class,"HTML name"),
			new SelectorItem(HSVSelector.class,"HSV"),
			new SelectorItem(PercentSelector.class,"Percent"),
			new SelectorItem(CMYSelector.class,"CMY"),
			new SelectorItem(CMYKSelector.class,"CMYK"),
			new SelectorItem(PaletteSelector.class,"Palette"),
	};
	
	public int[] indices;
	public boolean[] sel;
	
	private SelectorBar selectorBar;
	
	public void save()
	{
		IPath path = Activator.getDefault().getStateLocation();
		String filename = path.append("selectors.xml").toOSString();
		DialogSettings settings = new DialogSettings("selectors");
		try {
			settings.put("ids", saveIndices());
			settings.put("sel", saveSel());
			settings.save(filename);
		} catch (IOException e) {}
	}
	
	public void load()
	{
		IPath path = Activator.getDefault().getStateLocation();
		String filename = path.append("selectors.xml").toOSString();
		DialogSettings settings = new DialogSettings("selectors");
		try {
			settings.load(filename);
			loadIndices(settings.get("ids"));
			loadSel(settings.get("sel"));
		} catch (IOException e) {
		}
	}
	
	public String saveIndices()
	{
		StringBuffer b = new StringBuffer();
		for (int i=0;i<indices.length;i++)
		{
			b.append(indices[i]);
			if (i!=indices.length-1) b.append(',');
		}
		
		return b.toString();
	}
	
	public String saveSel()
	{
		StringBuffer b = new StringBuffer();
		for (int i=0;i<sel.length;i++)
		{
			b.append(sel[i]?1:0);
	        if (i!=sel.length-1) b.append(',');
		}

		return b.toString();
	}
	
	public void loadIndices(String s)
	{
		if (s==null) return;

		String[] n=s.split(",");
		for (int i=0;i<n.length;i++)
		{
			indices[i]=Integer.parseInt(n[i]);
		}
	}
	
	public void loadSel(String s)
	{
		if (s==null) return;
		
		String[] n=s.split(",");
		for (int i=0;i<n.length;i++)
		{
			sel[i]=(n[i].equals("1"));
		}
	}
	
	public SelectorManager(SelectorBar selectorBar)
	{
		this.selectorBar=selectorBar;
		
		indices=new int[allItems.length];
		sel=new boolean[allItems.length];
		for (int i=0;i<allItems.length;i++)
		{
			indices[i]=i;
			sel[i]=true;
		}
	}
	
	@SuppressWarnings("unchecked")
	public void applyToSelectorBar()
	{
		int insertToIndex=0;
		ColorSelector cs=null;
		
		for (int i=0;i<indices.length;i++)
		{
			if (sel[indices[i]]==true)
			{
				cs=selectorBar.getColorSelector(allItems[indices[i]].getSelector());
				
				if (cs==null)
				{
					Constructor<? extends ColorSelector> con=(Constructor<? extends ColorSelector>) allItems[indices[i]].getSelector().getConstructors()[0];
					try {
						cs=con.newInstance(selectorBar,allItems[indices[i]].getName());
					} catch (Exception e) {
					}
					cs.expand();
				}
				else
				{
					selectorBar.removeColorSelector(allItems[indices[i]].getSelector(),false);
				}
				selectorBar.addColorSelector(insertToIndex, cs);
				insertToIndex++;
			}
			else
			{
				selectorBar.removeColorSelector(allItems[indices[i]].getSelector(),true);
			}
		}

		selectorBar.refreshPositions();
		selectorBar.refreshAllSelectors();
	}
}

class SelectorItem
{
	private Class<? extends ColorSelector> selector;
	private String name;
	public Class<? extends ColorSelector> getSelector() {
		return selector;
	}

	public String getName() {
		return name;
	}

	public SelectorItem(Class<? extends ColorSelector> selector,String name) {
		this.selector = selector;
		this.name = name;
	}
}