package cz.wz.milanl.colors.views;

import org.eclipse.swt.widgets.ColorDialog;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.part.*;
import org.eclipse.ui.texteditor.IDocumentProvider;
import org.eclipse.ui.texteditor.ITextEditor;
import org.eclipse.swt.dnd.Clipboard;
import org.eclipse.swt.dnd.TextTransfer;
import org.eclipse.swt.events.DisposeEvent;
import org.eclipse.swt.events.DisposeListener;
import org.eclipse.swt.layout.FormAttachment;
import org.eclipse.swt.layout.FormData;
import org.eclipse.swt.layout.FormLayout;
import org.eclipse.jface.action.*;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.text.BadLocationException;
import org.eclipse.jface.text.IDocument;
import org.eclipse.jface.text.TextSelection;
import org.eclipse.ui.*;


import cz.wz.milanl.colors.organizer.Organizer;
import cz.wz.milanl.colors.organizer.OrganizerWindow;
import cz.wz.milanl.colors.selectors.*;
import cz.wz.milanl.util.Icons;
import cz.wz.milanl.util.ScreenSelector;

public class Colors extends ViewPart {
	private Composite parent;

	private ColorView colorView;
	
	private SelectorManager selectorManager;
	private Organizer organizer;
	
	private OrganizerWindow organizerWindow;
	
	private SelectorBar selectorBar;

	private Action pickFromScreen1;
	private Action pickFromScreen2;
	private Action showColorDialog;
	
	private Action editSelectors;
	private Action colorOrganizer;
	
	private ScreenSelector screenSelector;
	   
	public Colors() {
	}

	public void createPartControl(Composite parent) {

		parent.addDisposeListener(new DisposeListener()
		{
			public void widgetDisposed(DisposeEvent e) {
				end();
			}
			
		});
		this.parent = parent;
		parent.setLayout(new FormLayout());

		
		colorView = new ColorView(parent);

		selectorBar = new SelectorBar(parent, this, colorView);
		
		selectorManager=new SelectorManager(selectorBar);
		selectorManager.load();
		
		
		colorView.setSelectorBar(selectorBar);
		selectorManager.applyToSelectorBar();
		
		
		
		FormData data = new FormData();
		data.top = new FormAttachment(0, 20);
		data.bottom = new FormAttachment(100);
		data.left = new FormAttachment(0);
		data.right = new FormAttachment(100);
		selectorBar.setLayoutData(data);

		
		
		makeActions();
		contributeToActionBars();
	}

	

	private void contributeToActionBars() {
		IActionBars bars = getViewSite().getActionBars();
		fillLocalPullDown(bars.getMenuManager());
		fillLocalToolBar(bars.getToolBarManager());
	}

	private void fillLocalPullDown(IMenuManager manager) {
		manager.add(editSelectors);
		//manager.add(colorOrganizer);
	}

	private void fillLocalToolBar(IToolBarManager manager) {
		manager.add(pickFromScreen1);
		manager.add(pickFromScreen2);
		manager.add(showColorDialog);
	}

	private void makeActions() {
		pickFromScreen1 = new Action() {
			public void run() {
				activateScreenSelector();
			}
		};
		pickFromScreen1.setText("From screen");
		pickFromScreen1.setToolTipText("Pick a color from screen");
		pickFromScreen1.setImageDescriptor(ImageDescriptor.createFromImage(Icons.get("from_screen")));
		
		pickFromScreen2 = new Action() {
			public void run() {
				activateScreenSelector();
				parent.getShell().setMinimized(true);
			}
		};
		pickFromScreen2.setText("From screen (minimize)");
		pickFromScreen2.setToolTipText("Minimize Eclipse window and pick a color from screen");
		pickFromScreen2.setImageDescriptor(ImageDescriptor.createFromImage(Icons.get("from_screen_minimize")));

		showColorDialog = new Action() {
			public void run() {
				ColorDialog colorDialog=new ColorDialog(parent.getShell());
				colorDialog.setRGB(colorView.getColor());
				colorDialog.open();
				colorView.changeColor(colorDialog.getRGB(), null);
			}
		};
		showColorDialog.setText("Color dialog");
		showColorDialog.setToolTipText("Open color dialog");
		showColorDialog.setImageDescriptor(ImageDescriptor.createFromImage(Icons.get("color_dialog")));
		
		editSelectors = new Action() {
			public void run() {
				EditSelectorsWindow esw=new EditSelectorsWindow(selectorManager,getSite().getWorkbenchWindow().getShell(),0);
				esw.open();
			}
		};
		editSelectors.setText("Edit selectors");
		
		colorOrganizer = new Action() {
			public void run() {
				if (organizerWindow==null)
				{
					organizer=new Organizer();
					organizer.load();
					
					organizerWindow=new OrganizerWindow(organizer,getSite().getWorkbenchWindow().getShell(),0,colorView);
				}
				organizerWindow.open();
			}
		};
		colorOrganizer.setText("Color organizer");
	}

	public void setFocus() {
		colorView.setFocus();
	}

	public void copyToClipboard(String text) {
		new Clipboard(Display.getDefault()).setContents(new String[] { text },
				new TextTransfer[] { TextTransfer.getInstance() });
	}

	public String getFromClipboard() {
		return (String) new Clipboard(Display.getDefault())
				.getContents(TextTransfer.getInstance());
	}

	public void clearClipboard()
	{
		Clipboard c=new Clipboard(Display.getDefault());
		c.setContents(new String[] { "." },
				new TextTransfer[] { TextTransfer.getInstance() });
		c.clearContents();
	}
	
	public void copyToSource(String text) {
		if (getSourceSelection().equals(text))
			return;
		ITextEditor te = (ITextEditor) getViewSite().getWorkbenchWindow()
				.getActivePage().getActiveEditor();
		if (te==null) return;
		IDocumentProvider dp = te.getDocumentProvider();
		IDocument doc = dp.getDocument(te.getEditorInput());
		TextSelection selection = (TextSelection) te.getSelectionProvider()
				.getSelection();

		try {
			int off = selection.getOffset();
			doc.replace(selection.getOffset(), selection.getLength(), text);
			te.getSelectionProvider().setSelection(
					new TextSelection(off, text.length()));
		} catch (BadLocationException e1) {

		}
	}

	public String getSourceSelection() {
		IEditorPart editor = getViewSite().getWorkbenchWindow().getActivePage()
				.getActiveEditor();
		if (editor==null) return "";
		ITextEditor te = (ITextEditor) editor;
		return ((TextSelection) te.getSelectionProvider().getSelection())
				.getText();
	}
	
	private void activateScreenSelector()
	{
		screenSelector=new ScreenSelector(colorView, parent.getShell());
	}
	
	private void exitScreenSelector()
	{
		if (screenSelector!=null)
		{
			screenSelector.end();
			screenSelector=null;
		}
	}
	
	private void end()
	{
		selectorBar.disableAutoClipboardActions();
		selectorBar.disableAutoSourceActions();
		exitScreenSelector();
	}
}